import 'core.dart';

class M4eExceptionCode {
  final String _exceptionCode;

  const M4eExceptionCode._(this._exceptionCode);

  static const kNoAuthUserExceptionCode = M4eExceptionCode._('NO_AUTH_USER');
  static const kNoAuthUserCredentialExceptionCode =
      M4eExceptionCode._('NO_CACHED_AUTH_CREDENTIAL');
  static const kNoCacheDataExceptionCode = M4eExceptionCode._('NO_CACHE_DATA');
  static const kFailedTransactionExceptionCode =
      M4eExceptionCode._('FAILED_TRANSACTION');
  static const kUndefinedExceptionCode = M4eExceptionCode._('UNDEFINED');
  static const kConnectionTimeoutExceptionCode =
      M4eExceptionCode._('CONNECTION_TIMEOUT');
  static const kNoInternetConnectionExceptionCode =
      M4eExceptionCode._('NO_INTERNET_CONNECTION');
  static const kNotFoundExceptionCode = M4eExceptionCode._('NOT_FOUND');

  static M4eExceptionCode parse(String type) {
    return M4eExceptionCode._(type);
  }

  @override
  String toString() => _exceptionCode;

  bool operator ==(o) =>
      (o is M4eExceptionCode) && o._exceptionCode == this._exceptionCode;

  @override
  int get hashCode => _exceptionCode.hashCode;
}

class M4eExceptionMessages {
  static final kNoAuthUser = M4eAuthException(
    code: M4eExceptionCode.kNoAuthUserExceptionCode,
    message: 'No authenticated m4e user account',
  );

  static final kNoAuthCredential = M4eAuthException(
    code: M4eExceptionCode.kNoAuthUserCredentialExceptionCode,
    message: 'No user authentication credential found',
  );

  static final kNoCacheDataException = M4eCacheException(
    code: M4eExceptionCode.kNoCacheDataExceptionCode,
    message: 'No cache data!',
  );

  static final kFailedTransactionException = M4eServerException(
    code: M4eExceptionCode.kFailedTransactionExceptionCode,
    message: 'Transaction failed to complete!',
  );

  static final kUndefinedException = M4eServerException(
    code: M4eExceptionCode.kUndefinedExceptionCode,
    message: 'Something went wrong!',
  );

  static final kConnectionTimeOutException = M4eServerException(
    code: M4eExceptionCode.kConnectionTimeoutExceptionCode,
    message: 'Connection timeout!',
  );

  static final kNoInternetConnectionException = M4eServerException(
    code: M4eExceptionCode.kNoInternetConnectionExceptionCode,
    message: 'No internet connection!',
  );

  static final kNoNotFoundException = M4eServerException(
    code: M4eExceptionCode.kNotFoundExceptionCode,
    message: 'Record not found!',
  );
}

class M4eClientRequest {
  /// connection timeout
  static const kConnectionTimeout = 60000;
  static const kRecieveTimeout = 60000;
}

class M4ePersistence {
  static const kM4eAuthToken = 'M4E_AUTH_TOKEN';
  static const kM4eAuthCredential = 'M4E_AUTH_CREDENTIAL';
  static const kM4eUserWallets = 'M4E_USER_WALLETS';
  static const kM4eUserTransactionLogs = 'M4E_USER_TRANSACTION_LOGS';
}

class M4eCurrency {
  static const kGHS = 'GHS';
}

class M4eAccessPermissionType {
  final String _type;

  const M4eAccessPermissionType._(this._type);

  static const ENTITY_ENTITY = M4eAccessPermissionType._('ENTITY_ENTITY');
  static const ENTITY_WALLET = M4eAccessPermissionType._('ENTITY_WALLET');
  static const WALLET_WALLET = M4eAccessPermissionType._('WALLET_WALLET');

  factory M4eAccessPermissionType.parse(String type) {
    return M4eAccessPermissionType._(type);
  }

  @override
  bool operator ==(Object o) {
    if (identical(this, o)) return true;

    return o is M4eAccessPermissionType && o._type == _type;
  }

  @override
  int get hashCode => _type.hashCode;

  @override
  String toString() => '$_type';
}
